==> distfun-0.3-1-src.zip: distfun/src/cdflib/bgrat.c <==
/* bgrat.f -- translated by f2c (version 20100827).
You must link the resulting object file with libf2c:
on Microsoft Windows system, link with libf2c.lib;
on Linux or Unix systems, link with .../path/to/libf2c.a -lm
or, if you install libf2c.a in a standard place, with -lf2c -lm
-- in that order, at the end of the command line, as in
cc *.o -lf2c -lm
Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

http://www.netlib.org/f2c/libf2c.zip
*/

#include "cdflib.h"

/* Subroutine */ int bgrat_(double *a, double *b, double *x, 
	double *y, double *w, double *eps, int *ierr)
{
	/* System generated locals */
	int i__1;
	double d__1;

	/* Builtin functions */
	double log(double), exp(double);

	/* Local variables */
	static double c__[30], d__[30];
	static int i__;
	static double j, l;
	static int n;
	static double p, q, r__, s, t, u, v, z__, n2, t2, dj, cn, nu, bm1;
	static int nm1;
	static double lnx, sum;
	static double bp2n, coef;

	/* ----------------------------------------------------------------------- */
	/*     ASYMPTOTIC EXPANSION FOR IX(A,B) WHEN A IS LARGER THAN B. */
	/*     THE RESULT OF THE EXPANSION IS ADDED TO W. IT IS ASSUMED */
	/*     THAT A .GE. 15 AND B .LE. 1.  EPS IS THE TOLERANCE USED. */
	/*     IERR IS A VARIABLE THAT REPORTS THE STATUS OF THE RESULTS. */
	/* ----------------------------------------------------------------------- */
	/*     .. Scalar Arguments .. */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */
	/*     .. Local Arrays .. */
	/*     .. */

	/*     .. */

	/*     .. */
	/*     .. Intrinsic Functions .. */
	/*     .. */
	/*     .. Executable Statements .. */

	bm1 = *b - .5 - .5;
	nu = *a + bm1 * .5;
	if (*y > .375) {
		goto L10;
	}
	d__1 = -(*y);
	lnx = alnrel_(&d__1);
	goto L20;
L10:
	lnx = log(*x);
L20:
	z__ = -nu * lnx;
	if (*b * z__ == 0.) {
		goto L70;
	}

	/*                 COMPUTATION OF THE EXPANSION */
	/*                 SET R = EXP(-Z)*Z**B/GAMMA(B) */

	r__ = *b * (gam1_(b) + 1.) * exp(*b * log(z__));
	r__ = r__ * exp(*a * lnx) * exp(bm1 * .5 * lnx);
	u = algdiv_(b, a) + *b * log(nu);
	u = r__ * exp(-u);
	if (u == 0.) {
		goto L70;
	}
	grat1_(b, &z__, &r__, &p, &q, eps);

	/* Computing 2nd power */
	d__1 = 1. / nu;
	v = d__1 * d__1 * .25;
	t2 = lnx * .25 * lnx;
	l = *w / u;
	j = q / r__;
	sum = j;
	t = 1.;
	cn = 1.;
	n2 = 0.;
	for (n = 1; n <= 30; ++n) {
		bp2n = *b + n2;
		j = (bp2n * (bp2n + 1.) * j + (z__ + bp2n + 1.) * t) * v;
		n2 += 2.;
		t *= t2;
		cn /= n2 * (n2 + 1.);
		c__[n - 1] = cn;
		s = 0.;
		if (n == 1) {
			goto L40;
		}
		nm1 = n - 1;
		coef = *b - n;
		i__1 = nm1;
		for (i__ = 1; i__ <= i__1; ++i__) {
			s += coef * c__[i__ - 1] * d__[n - i__ - 1];
			coef += *b;
			/* L30: */
		}
L40:
		d__[n - 1] = bm1 * cn + s / n;
		dj = d__[n - 1] * j;
		sum += dj;
		if (sum <= 0.) {
			goto L70;
		}
		if (fabs(dj) <= *eps * (sum + l)) {
			goto L60;
		}
		/* L50: */
	}

	/*                    ADD THE RESULTS TO W */

L60:
	*ierr = 0;
	*w += u * sum;
	return 0;

	/*               THE EXPANSION CANNOT BE COMPUTED */

L70:
	*ierr = 1;
	return 0;
} /* bgrat_ */

==> distfun-0.5-1-src.zip: distfun/src/cdflib/bgrat.c <==
/* bgrat.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include <math.h>

#include "cdflib.h"
#include "cdflib_private.h" 

/* Subroutine */ int cdflib_bgrat(double a, double b, double x, 
	double y, double *w, double eps, int *ierr)
{
	/* System generated locals */
	int i__1;
	double d__1;

	/* Local variables */
	static double c__[30], d__[30];
	static int i__;
	static double j, l;
	static int n;
	static double p, q, r__, s, t, u, v, z__, n2, t2, dj, cn, nu, bm1;
	static int nm1;
	static double lnx, sum;
	static double bp2n, coef;

	/* ----------------------------------------------------------------------- */
	/*     ASYMPTOTIC EXPANSION FOR IX(A,B) WHEN A IS LARGER THAN B. */
	/*     THE RESULT OF THE EXPANSION IS ADDED TO W. IT IS ASSUMED */
	/*     THAT A .GE. 15 AND B .LE. 1.  EPS IS THE TOLERANCE USED. */
	/*     IERR IS A VARIABLE THAT REPORTS THE STATUS OF THE RESULTS. */
	/* ----------------------------------------------------------------------- */

	bm1 = b - .5 - .5;
	nu = a + bm1 * .5;
	if (y > .375) {
		goto L10;
	}
	d__1 = -(y);
	lnx = cdflib_log1p(d__1);
	goto L20;
L10:
	lnx = log(x);
L20:
	z__ = -nu * lnx;
	if (b * z__ == 0.) {
		goto L70;
	}

	/*                 COMPUTATION OF THE EXPANSION */
	/*                 SET R = EXP(-Z)*Z**B/GAMMA(B) */

	r__ = b * (cdflib_gam1(b) + 1.) * exp(b * log(z__));
	r__ = r__ * exp(a * lnx) * exp(bm1 * .5 * lnx);
	u = cdflib_algdiv(b, a) + b * log(nu);
	u = r__ * exp(-u);
	if (u == 0.) {
		goto L70;
	}
	cdflib_grat1(b, z__, r__, &p, &q, eps);

	/* Computing 2nd power */
	d__1 = 1. / nu;
	v = d__1 * d__1 * .25;
	t2 = lnx * .25 * lnx;
	l = *w / u;
	j = q / r__;
	sum = j;
	t = 1.;
	cn = 1.;
	n2 = 0.;
	for (n = 1; n <= 30; ++n) {
		bp2n = b + n2;
		j = (bp2n * (bp2n + 1.) * j + (z__ + bp2n + 1.) * t) * v;
		n2 += 2.;
		t *= t2;
		cn /= n2 * (n2 + 1.);
		c__[n - 1] = cn;
		s = 0.;
		if (n == 1) {
			goto L40;
		}
		nm1 = n - 1;
		coef = b - n;
		i__1 = nm1;
		for (i__ = 1; i__ <= i__1; ++i__) {
			s += coef * c__[i__ - 1] * d__[n - i__ - 1];
			coef += b;
			/* L30: */
		}
L40:
		d__[n - 1] = bm1 * cn + s / n;
		dj = d__[n - 1] * j;
		sum += dj;
		if (sum <= 0.) {
			goto L70;
		}
		if (fabs(dj) <= eps * (sum + l)) {
			goto L60;
		}
		/* L50: */
	}

	/*                    ADD THE RESULTS TO W */

L60:
	*ierr = 0;
	*w += u * sum;
	return 0;

	/*               THE EXPANSION CANNOT BE COMPUTED */

L70:
	*ierr = 1;
	return 0;
}
